package com.elitesland.security;

import com.elitesland.system.entity.RoleDO;
import com.elitesland.system.repo.PermissionRepo;
import com.elitesland.system.repo.RoleRepo;
import lombok.val;
import org.springframework.security.access.ConfigAttribute;
import org.springframework.security.access.SecurityConfig;
import org.springframework.security.web.FilterInvocation;
import org.springframework.security.web.access.intercept.FilterInvocationSecurityMetadataSource;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import reactor.util.function.Tuples;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.stream.Collectors;

/**
 * <pre>
 * 动态鉴权的核心，根据访问的URL，获取对应的角色列表
 * </pre>
 *
 * @author Mir
 * @date 2020/6/15
 */
public class DynamicFilterInvocationSecurityMetadataSource implements FilterInvocationSecurityMetadataSource {

    @Resource
    private RequestMatcherCreator requestMatcherCreator;

    @Resource
    private PermissionRepo permissionRepo;

    @Resource
    private RoleRepo roleRepo;

    @Override
    public Collection<ConfigAttribute> getAttributes(Object object) throws IllegalArgumentException {
        val request = ((FilterInvocation) object).getRequest();

        val permissions = permissionRepo.findAll();
        val metaResources = permissions.stream()
                .map(p -> Tuples.of(p.getPattern(), request.getMethod()))
                .collect(Collectors.toSet());
        val requestMatchers = requestMatcherCreator.convertToRequestMatcher(metaResources);

        val reqMatcher = requestMatchers.stream().filter(m -> m.matches(request))
                .findAny();
        if (reqMatcher.isPresent()) {
            val antPathRequestMatcher = (AntPathRequestMatcher) reqMatcher.get();

            val pattern = antPathRequestMatcher.getPattern();
            val permRoleCodes = permissions.stream().filter(p -> p.getPattern().equals(pattern))
                    .flatMap(p -> p.getRoles().stream().map(RoleDO::getCode))
                    .toArray(String[]::new);
            return SecurityConfig.createList(permRoleCodes);
        }

        return SecurityConfig.createList();
    }

    @Override
    public Collection<ConfigAttribute> getAllConfigAttributes() {
        val roleList = roleRepo.findAll();
        val roleCodeList = roleList.stream().map(RoleDO::getCode).toArray(String[]::new);
        return SecurityConfig.createList(roleCodeList);
    }

    @Override
    public boolean supports(Class<?> clazz) {
        return FilterInvocation.class.isAssignableFrom(clazz);
    }
}
