package com.elitesland.system.aop;

import com.elitesland.core.base.PagingVO;
import com.elitesland.system.annotation.SysCode;
import com.elitesland.system.service.CatCodeService;
import lombok.AllArgsConstructor;
import lombok.val;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <pre>
 *     针对数据库返回结果集进行切面处理UDC的转换
 * </pre>
 * @author Mir
 * @date 2020-04-20
 */

@Aspect
@Component
@AllArgsConstructor
public class SysCodeAop {

    private final CatCodeService catCodeService;

    @Pointcut("@annotation(com.elitesland.system.annotation.SysCodeProc)")
    public void sysCodeCut() {
    }

    @Around("sysCodeCut()")
    public Object around(ProceedingJoinPoint point) throws Throwable {
        val result = point.proceed();

        if (result != null) {
            if (result instanceof PagingVO) {
                val resultList = (PagingVO<?>) result;
                for (int i = 0; i < resultList.getRecords().size(); i++) {
                    Object oi = resultList.getRecords().get(i);
                    translate(oi);
                }
            }
            if (result instanceof List) {
                val resultList = (List<?>) result;
                for (Object o : resultList) {
                    translate(o);
                }
            } else {
                translate(result);
            }
        }

        return result;
    }

    private void translate(Object oi) throws InvocationTargetException, IllegalAccessException {
        Class<?> c = oi.getClass();
        Class<?>[] types = {String.class};
        val fieldNames = Arrays.stream(c.getDeclaredFields()).map(Field::getName).collect(Collectors.toList());
        for (Field field : c.getDeclaredFields()) {
            if (field.getType() != String.class && field.isAnnotationPresent(SysCode.class)) {
                try {
                    val getter = c.getMethod("get" + StringUtils.capitalize(field.getName()));
                    val o = getter.invoke(oi);

                    translate(o);
                } catch (NoSuchMethodException ignored) {
                }
            }

            if (field.isAnnotationPresent(SysCode.class)) {
                if (fieldNames.contains(field.getName() + "Name")) {
                    try {
                        val anno = field.getAnnotation(SysCode.class);
                        val codeMap = catCodeService.getCodeMap(anno.sys(), anno.mod());

                        val fieldMethodName = StringUtils.capitalize(field.getName());
                        val getter = c.getMethod("get" + fieldMethodName);
                        String code = (String) getter.invoke(oi);
                        val desc = codeMap.getOrDefault(code, "");
                        val m = c.getMethod("set" + fieldMethodName + "Name", types);
                        m.invoke(oi, desc);
                    } catch (NoSuchMethodException ignored) {
                    }
                }
            }
        }
    }
}
