package com.elitesland.system.controller;

import com.elitesland.system.convert.UserConvert;
import com.elitesland.system.entity.UserDO;
import com.elitesland.system.repo.UserRepo;
import com.elitesland.system.vo.UserVO;
import io.swagger.annotations.Api;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import lombok.val;
import org.springframework.http.ResponseEntity;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <pre>
 * 系统用户账号维护接口
 * </pre>
 *
 * @author Mir
 * @date 2020/6/15
 */
@Slf4j
@RestController
@RequestMapping("/sys/users")
@RequiredArgsConstructor
@Api(value = "系统用户账号维护", tags = {"系统用户账号维护"})
public class UserController {

    private final UserRepo userRepo;

    private final PasswordEncoder passwordEncoder;

    @GetMapping
    public ResponseEntity<List<UserVO>> listAll() {
        val userVos = userRepo.findAll().stream().map(UserConvert.INSTANCE::doToVo).collect(Collectors.toList());
        return ResponseEntity.ok(userVos);
    }

    @PostMapping
    public ResponseEntity<Long> create(@RequestBody UserDO userDO) {
        userDO.setPassword(passwordEncoder.encode(userDO.getPassword()));
        val u = userRepo.save(userDO);
        return ResponseEntity.ok(u.getId());
    }
}
