package com.elitesland.system.entity;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.data.annotation.CreatedBy;
import org.springframework.data.annotation.CreatedDate;
import org.springframework.data.annotation.LastModifiedBy;
import org.springframework.data.annotation.LastModifiedDate;
import org.springframework.data.jpa.domain.support.AuditingEntityListener;

import javax.persistence.*;
import javax.validation.constraints.Null;
import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * <p>
 * 分类码明细表
 * </p>
 *
 * @author Mir
 * @date 2020/5/15
 */

@Entity
@Table(name = "y0055")
@org.hibernate.annotations.Table(appliesTo = "y0055", comment = "分类码明细记录，对应分类码定义表的分类集合")
@Data
@ApiModel(value = "分类码明细", description = "分类码明细")
@EntityListeners(AuditingEntityListener.class)
public class  CatCodeDO implements Serializable {

    private static final long serialVersionUID = 407028412678884366L;

    @EmbeddedId
    private CatCodeKey key;

    @ApiModelProperty("描述、说明")
    @Column(columnDefinition = "varchar(32) default '' comment '描述、说明'", nullable = true)
    private String description;

    @ApiModelProperty("是否硬编码")
    @Column(columnDefinition = "tinyint(1) default true comment '是否硬编码'")
    private Boolean isHD;

    @ApiModelProperty("预留整数01")
    @Column(columnDefinition = "int default 0 comment '预留整数01'")
    private Integer int01;

    @ApiModelProperty("预留整数02")
    @Column(columnDefinition = "int default 0 comment '预留整数02'")
    private Integer int02;

    @ApiModelProperty("预留整数03")
    @Column(columnDefinition = "int default 0 comment '预留整数03'")
    private Integer int03;

    @ApiModelProperty("预留整数04")
    @Column(columnDefinition = "int default 0 comment '预留整数04'")
    private Integer int04;

    @ApiModelProperty("预留浮点数01")
    @Column(columnDefinition = "float(18, 4) default 0.0 comment '预留浮点数01'")
    private Double double01;

    @ApiModelProperty("预留浮点数02")
    @Column(columnDefinition = "float(18, 4) default 0.0 comment '预留浮点数02'")
    private Double double02;

    @ApiModelProperty("预留浮点数03")
    @Column(columnDefinition = "float(18, 4) default 0.0 comment '预留浮点数03'")
    private Double double03;

    @ApiModelProperty("预留浮点数04")
    @Column(columnDefinition = "float(18, 4) default 0.0 comment '预留浮点数04'")
    private Double double04;

    @ApiModelProperty("预留字符串01")
    @Column(columnDefinition = "varchar(128) default '' comment '预留字符串01'")
    private String string01;

    @ApiModelProperty("预留字符串02")
    @Column(columnDefinition = "varchar(128) default '' comment '预留字符串02'")
    private String string02;

    @ApiModelProperty("预留字符串03")
    @Column(columnDefinition = "varchar(128) default '' comment '预留字符串03'")
    private String string03;

    @ApiModelProperty("预留字符串04")
    @Column(columnDefinition = "varchar(128) default '' comment '预留字符串04'")
    private String string04;

    @ApiModelProperty("预留日期时间01")
    @Column(columnDefinition = "datetime default NULL comment '预留日期时间01'")
    private LocalDateTime datetime01;

    @ApiModelProperty("预留日期时间02")
    @Column(columnDefinition = "datetime default NULL comment '预留日期时间02'")
    private LocalDateTime datetime02;

    @ApiModelProperty("预留日期时间03")
    @Column(columnDefinition = "datetime default NULL comment '预留日期时间03'")
    private LocalDateTime datetime03;

    @ApiModelProperty("预留日期时间04")
    @Column(columnDefinition = "datetime default NULL comment '预留日期时间04'")
    private LocalDateTime datetime04;

    @ApiModelProperty("预留开关01")
    @Column(columnDefinition = "tinyint(1) default NULL comment '预留开关01'")
    private Boolean flag01;

    @CreatedBy
    @Column(columnDefinition = "bigint default 0 comment '记录创建者ID'")
    private Long creator;

    @CreatedDate
    @Column(columnDefinition = "timestamp default NULL comment '记录创建时间'")
    private LocalDateTime created;

    @LastModifiedBy
    @Column(columnDefinition = "bigint default 0 comment '记录最后更新者ID'")
    private Long updater;

    @LastModifiedDate
    @Column(columnDefinition = "timestamp default NULL comment '记录最后更新时间'")
    private LocalDateTime updated;

    @ApiModelProperty(value = "逻辑删除，0：未删除，1：已删除")
    @Null(message = "逻辑删除不用传")
    @Column(columnDefinition = "int default 0 comment '逻辑删除，0：未删除，1：已删除'")
    private Integer deleted;

    @ApiModelProperty(value = "版本")
    @Null(message = "版本不用传")
    @Column(columnDefinition = "int default 0 comment '版本信息，前端不用传'")
    private Integer version;

}
