package com.elitesland.system.entity;

import com.elitesland.core.base.BaseModel;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Objects;
import java.util.Set;

/**
 * <pre>
 * [功能说明]
 * </pre>
 *
 * @author Mir
 * @date 2020/6/15
 */
@Entity
@Table(name = "sys_permission")
@org.hibernate.annotations.Table(appliesTo = "sys_permission", comment = "系统权限表，包括URL、操作")
@Data
@Accessors(chain = true)
@ApiModel(value = "系统权限表", description = "系统权限表，包括URL、操作")
public class PermissionDO extends BaseModel implements Serializable {
    private static final long serialVersionUID = 6810626977765149177L;

    @ApiModelProperty(value = "权限代码")
    @Column(columnDefinition = "varchar(16) default '' comment '权限代码'", nullable = false, unique = true)
    private String code;

    @ApiModelProperty(value = "权限名称")
    @Column(columnDefinition = "varchar(16) default '' comment '权限名称'", nullable = false)
    private String name;

    @ApiModelProperty(value = "权限类型")
    @Column(columnDefinition = "int default 0 comment '权限类型'")
    private Integer permType;

    @ApiModelProperty(value = "父权限ID")
    @Column(columnDefinition = "int default 0 comment '父权限ID'")
    private Integer parentId;

    @ApiModelProperty(value = "权限层级路径")
    @Column(columnDefinition = "varchar(256) default '' comment '权限层级路径'")
    private String path;

    @ApiModelProperty(value = "权限层级")
    @Column(columnDefinition = "int default 0 comment '权限层级'")
    private Integer level;

    @ApiModelProperty(value = "URL路径模式")
    @Column(columnDefinition = "varchar(256) default '' comment 'URL路径模式'")
    private String pattern;

    @ManyToMany(mappedBy = "permissions", cascade = CascadeType.ALL, fetch = FetchType.EAGER)
    @JsonIgnoreProperties({"permissions"})
    private Set<RoleDO> roles;

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof PermissionDO)) {
            return false;
        }
        if (!super.equals(o)) {
            return false;
        }
        PermissionDO entity = (PermissionDO) o;
        return getId().equals(entity.getId());
    }

    @Override
    public int hashCode() {
        return Objects.hash(getId());
    }
}