package com.elitesland.system.service;

import com.elitesland.system.entity.UserDO;
import com.elitesland.system.vo.UserVO;
import org.springframework.security.core.userdetails.UserDetails;

import java.util.Optional;

/**
 * <pre>
 * 系统用户操作逻辑服务类
 * </pre>
 *
 * @author Mir
 * @date 2020/6/16
 */
public interface UserService {

    /**
     * 根据用户账号，返回所有的用户信息，包括密码和角色清单
     *
     * @param username 用户账号
     * @return 对应的用户信息
     */
    Optional<UserDO> getUserByUsernameSec(String username);

    /**
     * 根据用户账号，返回脱敏的用户信息，VO
     *
     * @param username 用户账号
     * @return 脱敏的用户信息
     */
    Optional<UserVO> getUserByUsername(String username);

    /**
     * 判断对应用户账号是否存在
     *
     * @param username 用户账号
     * @return 是否存在
     */
    boolean isUserExists(String username);

    /**
     * 根据用户账号，返回对应Spring Security用UserDetails对象
     *
     * @param username 用户账号
     * @return UserDetails信息
     */
    Optional<UserDetails> loadUserDetailsByUsername(String username);
}
