package com.elitesland.system.service.impl;

import com.elitesland.core.base.PagingVO;
import com.elitesland.system.convert.CatCodeConvert;
import com.elitesland.system.entity.CatCodeDO;
import com.elitesland.system.entity.CatCodeKey;
import com.elitesland.system.entity.QCatCodeDO;
import com.elitesland.system.param.CatCodeQueryParam;
import com.elitesland.system.repo.CatCodeRepo;
import com.elitesland.system.repo.CatCodeRepoProc;
import com.elitesland.system.service.CatCodeService;
import com.elitesland.system.vo.CatCodeComboVo;
import com.querydsl.jpa.impl.JPAQueryFactory;
import lombok.RequiredArgsConstructor;
import lombok.val;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * <pre>
 * [功能说明]
 * </pre>
 *
 * @author Mir
 * @date 2020/6/25
 */
@Service
@RequiredArgsConstructor
public class CatCodeServiceImpl implements CatCodeService {

    private final CatCodeRepo catCodeRepo;

    private final JPAQueryFactory jpaQueryFactory;

    @Override
    public PagingVO<CatCodeDO> search(CatCodeQueryParam param) {
        val records = catCodeRepo.findAll(CatCodeRepoProc.where(param), param.getPageRequest());

        return PagingVO.<CatCodeDO>builder()
                .total(records.getTotalElements())
                .records(records.getContent())
                .build();
    }

    @Override
    public Optional<CatCodeDO> findOne(CatCodeKey key) {
        val code = QCatCodeDO.catCodeDO;

        return catCodeRepo.findOne(code.key.sysCode.eq(key.getSysCode())
                .and(code.key.modCode.eq(key.getModCode())
                        .and(code.key.catCode.eq(key.getCatCode()))));
    }

    @Override
    public void createCode(CatCodeDO code) {
        catCodeRepo.save(code);
    }

    @Override
    public void updateCode(CatCodeDO code) {
        val codeOpt = findOne(code.getKey());
        codeOpt.ifPresent(c -> {
                    code.setCreator(c.getCreator());
                    code.setCreated(c.getCreated());

                    catCodeRepo.save(code);
                }
        );
    }

    @Override
    @Cacheable(value = "catcodecombos", key = "targetClass + '-' + #sys + '-' + #mode")
    public List<CatCodeComboVo> listCodeCombos(String sys, String mode) {
        val catCode = QCatCodeDO.catCodeDO;

        val codes = jpaQueryFactory.select(catCode)
                .from(catCode)
                .where(catCode.key.sysCode.eq(sys)  )
                .where(catCode.key.modCode.eq(mode)).fetch();

        return codes.stream().map(CatCodeConvert.INSTANCE::codeToComboVo).collect(Collectors.toList());
    }

    @Override
    public Map<String, String> getCodeMap(String sys, String mod) {
        return null;
    }
}
