package com.elitesland.system.service.impl;

import com.elitesland.system.convert.UserConvert;
import com.elitesland.system.entity.QUserDO;
import com.elitesland.system.entity.RoleDO;
import com.elitesland.system.entity.UserDO;
import com.elitesland.system.repo.UserRepo;
import com.elitesland.system.service.UserService;
import com.elitesland.system.vo.UserVO;
import lombok.RequiredArgsConstructor;
import lombok.val;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Service;

import java.util.Optional;

/**
 * <pre>
 * [功能说明]
 * </pre>
 *
 * @author Mir
 * @date 2020/6/16
 */
@Service
@RequiredArgsConstructor
public class UserServiceImpl implements UserService {

    private final UserRepo userRepo;

    @Override
    public Optional<UserDO> getUserByUsernameSec(String username) {
        val userModel = QUserDO.userDO;
        val predicate = userModel.username.eq(username);
        return userRepo.findOne(predicate);
    }

    @Override
    public Optional<UserVO> getUserByUsername(String username) {
        val userOpt = getUserByUsernameSec(username);

        return userOpt.map(UserConvert.INSTANCE::doToVo);
    }

    @Override
    public boolean isUserExists(String username) {
        val userOpt = getUserByUsernameSec(username);
        return userOpt.isPresent();
    }

    @Override
    public Optional<UserDetails> loadUserDetailsByUsername(String username) {
        val userOpt = getUserByUsernameSec(username);
        return userOpt.map(o -> User.builder()
                .username(o.getUsername())
                .password(o.getPassword())
                .authorities(AuthorityUtils.createAuthorityList(o.getRoles().stream().map(RoleDO::getCode).toArray(String[]::new)))
                .build());
    }
}
